/*
	massiveDIS.hh
*/

#pragma once

#include <cmath>
#include "PDFinterface.hh"
#include "common.hh"

class MassiveDIS
{

public:

  MassiveDIS(const process p, const parton, std::string pdfset, int member=0);
  ~MassiveDIS();

  // masses in GeV
  inline void SetM1(const double M) {fM1 = M;};
  inline void SetM2(const double M) {fM2 = M;};
  inline double GetM1() {return fM1;};
  inline double GetM2() {return fM2;};
  inline void SetProcess(const process p, const parton part){ fProcess=p; fParton=part; InitCouplings(p, part); };
  inline void SetCounting(counting c=standard) {fCounting = c;};
  inline void SetDamping(bool d = true) {fDamping = d;}
  inline void SetPrec(double p) {fPrec = p;}
  void ReInitPDF(std::string PDFset, int member=0);

  inline PDFinterface::PDFs * GetPDF() {return fPDF;};

  inline double GetPlusF1(double x, const double Q) {
    if (x == 1) return GetPlusF1inOne(Q);
    return (1.-x)*(s1hat(x,Q)/8./shat(x,Q))*1/N1(Q)*fhat1(x,Q);
  }
  inline double GetPlusF2(double x, const double Q) {
    if (x == 1) return GetPlusF2inOne(Q);
    return (1.-x)*(s1hat(x,Q)/8./shat(x,Q))*1/N2(x,Q)*fhat2(x,Q);
  }
  inline double GetPlusF3(double x, const double Q) {
    if (x == 1) return GetPlusF3inOne(Q);
    return (1.-x)*(s1hat(x,Q)/8./shat(x,Q))*1/N3(x,Q)*fhat3(x,Q);
  }

  double F(const double x, const double Q, order, scheme, sfunction, const double muF_Q_ratio = 1, const double muR_Q_ratio = 1);

  double DeltaF2IC(const double, const double, const double, order);

private:
  MassiveDIS( MassiveDIS const&);
  MassiveDIS& operator=(MassiveDIS const&);
	
  process fProcess;
  parton fParton;
  PDFinterface::PDFs * fPDF;

  double fMuRQ;
  double fMuFQ;
	
  double fM1;
  double fM2;

  double fPrec;

  //couplings
  double fSp;
  double fSm;
  double fRm;
  double fRp;

  //gluon couplings
  double fQp;
  double fQm;

  counting fCounting;
  bool fDamping;

  void InitCouplings(const process, const parton);

  double F_g     (const double, const double, order, sfunction);
  double F_g_sub (const double, const double, order, sfunction);
  double F_g0    (const double, const double, order, sfunction);
  double F_g_sub0(const double, const double, order, sfunction);

  double F_q     (const double, const double, order, sfunction);
  double F_q_sub (const double, const double, order, sfunction);
  double F_q0    (const double, const double, order, sfunction);
  double F_q_sub0(const double, const double, order, sfunction);

  double prefactor_massive (const double, const double, sfunction);
  double prefactor_massless(const double, sfunction);

  double F_q_as(const double, const double, const double, sfunction, parton);
  double F_q_as_0(double, const double, const double, sfunction, parton); 

  double Fgluon0(double, const double, const double, sfunction fun);

  //Sigma_{pm,pm}
  inline double Sigmapp(const double Q) {return (pow(Q,2)+pow(fM2,2)+pow(fM1,2));}
  inline double Sigmapm(const double Q) {return (pow(Q,2)+pow(fM2,2)-pow(fM1,2));}
  inline double Sigmamp(const double Q) {return (pow(Q,2)-pow(fM2,2)+pow(fM1,2));}
  inline double Sigmamm(const double Q) {return (pow(Q,2)-pow(fM2,2)-pow(fM1,2));}

  //triangle function
  double DeltaFun(const double, const double, const double);

  double Delta(const double);
  double Deltaprime(double, const double);

  //scaling variable chi
  double chi(double, const double);
  double PlusPrescription(double, const double, double,  sfunction, parton); 

  /**************************LO**************************/

  inline double F1pref(const double Q) {return ((fSp*Sigmapp(Q)-2.*fM1*fM2*fSm)/2./Delta(Q));}
  inline double F2pref(const double x, const double Q) {return (fSp*Delta(Q)/2./pow(Q,2))*2.*x;}
  inline double F3pref(const double Q) {return 2.*fRp;}

  inline double F1masslesspref() {return fSp/2.;}
  inline double F2masslesspref(const double x) {return fSp*x;}
  inline double F3masslesspref() {return 2.*fRp;}
	
  /**************************NLO*************************/

  double DeltaTerms(const double, const sfunction);

  /*********************Delta terms********************/

  //I1
  double I1(double);

  //vertex correction coefficients
  double Cp(const double);
  double C1p(const double);
  double C1m(const double);
  double Cqm(const double);
  double Cqp(const double);
  double CRm_nopoles(const double);

  //soft real contributions
  double S1_nopoles(const double);
  double S2_nopoles(const double);
  double S3_nopoles(const double);

  //virtual contributions
  double V1_nopoles(const double);
  double V2_nopoles(const double);
  double V3_nopoles(const double);
  
  /****************Plus Distribution terms*************/

  //s1hat
  double s1hat(double, const double);
  double shat(double, const double);

  double Lxi(double, const double);
  double Ixi(double, const double);
  double IxiReg(double, const double);

  //normalisation factors
  double N1(const double);
  double N2(double, const double);
  double N3(double, const double);

  //massive matrix elements
  double fhat1(double, const double);
  double fhat2(double, const double);
  double fhat3(double, const double);
	
  inline double GetPlusF1inOne(const double Q) {return (1./s1hatReg(1,Q)/8./shat(1,Q))*1/N1(Q)*fhat1Reg(1,Q);}
  inline double GetPlusF2inOne(const double Q) {return (1./s1hatReg(1,Q)/8./shat(1,Q))*1/N2(1,Q)*fhat2Reg(1,Q);}
  inline double GetPlusF3inOne(const double Q) {return (1./s1hatReg(1,Q)/8./shat(1,Q))*1/N3(1,Q)*fhat3Reg(1,Q);}

  double s1hatReg(double, const double);

  double fhat1Reg(double, const double);
  double fhat2Reg(double, const double);
  double fhat3Reg(double, const double);

  //subtraction terms

  double q_sub(double, const double, const double, sfunction, parton);
  double g_sub(double, const double, const double, sfunction);

  //gluon contribution to Fi
	
  friend double gluon_coeff(double, double, MassiveDIS *, sfunction);
  double Fgluon(double, const double, const double, sfunction fun); 
	
  double gluon_f1(double, const double);
  double gluon_f2(double, const double);
  double gluon_f3(double, const double);

  double gluon_L(double, const double);
  double gluon_Ltilde(double, const double);
  double gluon_v(double, const double);
  double gluon_vbar(double, const double);

};

